@@ -643,6 +643,51 @@ class Agent(Insight, Channel, Catalog, Dataset):
         r.raise_for_status()
         return r.json()['id']
 
+
+    def add_user_to_channel(self, channel_slug, user_email, role="FOLLOWER"):
+
+        url = self._base_url_askdata +"/smartbot/share/principals?q="+ user_email
+        s = requests.Session()
+        s.keep_alive = False
+        retries = Retry(total=5, backoff_factor=1, status_forcelist=[502, 503, 504])
+        s.mount('https://', HTTPAdapter(max_retries=retries))
+        r = s.get(url=url, headers=self._headers)
+        r.raise_for_status()
+        if (r != None and r.json() != []):
+            user_id = r.json()[0]["id"]
+        else:
+            print("Please insert valid email")
+            return None
+
+        channel_id = self.get_channel(channel_slug)
+        self.__add_user_to_ch(channel_id, user_id, role)
+
+    '''
+    role: FOLLOWER/ADMIN
+    '''
+    def __add_user_to_ch(self, channel_id, user_id, role="FOLLOWER"):
+
+        data = {
+            "principals": [
+                {
+                    "id": user_id,
+                    "type": "USER"
+                }
+            ],
+            "permission": role
+        }
+
+        s = requests.Session()
+        s.keep_alive = False
+        retries = Retry(total=5, backoff_factor=1, status_forcelist=[502, 503, 504])
+        s.mount('https://', HTTPAdapter(max_retries=retries))
+
+        authentication_url = self._base_url + "/smartfeed/channels/{}/users".format(channel_id)
+        r = s.post(url=authentication_url, headers=self._headers, json=data)
+        r.raise_for_status()
+        #return r
+
+
     def get_channel(self, channel_code):
 
         s = requests.Session()
