@@ -37,7 +37,9 @@ class Field:
     entityType: Optional[str] = None
     alias: Optional[str] = None
     internalDataType: Optional[str] = None
-    originalDataType: Optional[str] = None
+    sourceDataType: Optional[str] = None
+    sqlExpression: Optional[str] = None
+    sourceFormat: Optional[str] = None
 
 
 class SQLOperator(Enum):
@@ -181,7 +183,7 @@ class SQLSorting(Enum):
 @dataclass
 class Sorting:
     field: str
-    order: Union[SQLSorting,str]
+    order: Union[SQLSorting, str]
 
 
 @dataclass
@@ -322,7 +324,10 @@ class Query:
                         operator = "<"
                 else:
                     operator = None
-                formatted_cond = where_condition.field.column + " (" + where_condition.field.internalDataType + ") " + operator + " " + formatted_value
+                if where_condition.field.sourceDataType is not None:
+                    formatted_cond = where_condition.field.column + " (" + where_condition.field.sourceDataType + ") " + operator + " " + formatted_value
+                else:
+                    formatted_cond = where_condition.field.column + " " + operator + " " + formatted_value
                 conditions.append(formatted_cond)
             elif isinstance(where_condition, CompositeCondition):
                 other_conds = self._where_spell_out(where_condition.conditions)
@@ -335,22 +340,97 @@ class Query:
     def spell_out(self):
         spell_str = '# {"fields":['
 
+        consistent_datatypes = {"numeric": ["int", "float", "double", "int32", "int64"],
+                                "date": ["date", "timestamp", "datetime"],
+                                "string": ["string", "varchar", "varchar2"]}
+
         fields = []
         for field in self.fields:
-            if field.alias is None:
-                if field.aggregation is not None and isinstance(field.aggregation, str):
-                    string = '"' + field.aggregation + " " + field.column + " (" + field.internalDataType + ")" + '"'
-                elif field.aggregation is not None and isinstance(field.aggregation, SQLFunction):
-                    string = '"' + field.aggregation.name + " " + field.column + " (" + field.internalDataType + ")" + '"'
+            # if there is sql expression it always wins
+            if field.sqlExpression is not None:
+                if field.alias is None:
+                    if field.aggregation is not None and isinstance(field.aggregation, str):
+                        string = '"' + field.aggregation + " " + field.sqlExpression + '"'
+                    elif field.aggregation is not None and isinstance(field.aggregation, SQLFunction):
+                        string = '"' + field.aggregation.name + " " + field.sqlExpression + '"'
+                    else:
+                        string = '"' + field.sqlExpression + '"'
                 else:
-                    string = '"' + field.column + " (" + field.internalDataType + ")" + '"'
+                    if field.aggregation is not None and isinstance(field.aggregation, str):
+                        string = '"' + field.aggregation + " " + field.sqlExpression + " AS " + field.alias + '"'
+                    elif field.aggregation is not None and isinstance(field.aggregation, SQLFunction):
+                        string = '"' + field.aggregation.name + " " + field.sqlExpression + " AS " + field.alias + '"'
+                    else:
+                        string = '"' + field.sqlExpression + " AS " + field.alias + '"'
             else:
-                if field.aggregation is not None and isinstance(field.aggregation, str):
-                    string = '"' + field.aggregation + " " + field.column + " (" + field.internalDataType + ") AS " + field.alias + '"'
-                elif field.aggregation is not None and isinstance(field.aggregation, SQLFunction):
-                    string = '"' + field.aggregation.name + " " + field.column + " (" + field.internalDataType + ") AS " + field.alias + '"'
+                # otherwise, check if internal and original data types are not consistent each other
+                consistent_vec = consistent_datatypes[field.internalDataType.lower()]
+                if field.internalDataType.lower() == "numeric" and field.sourceDataType.lower() not in consistent_vec:
+                    # if internal and original data types are not consistent each other use this pattern
+                    # aggregation column (original data type, to internal data type ) AS Alias
+                    if field.alias is None:
+                        if field.aggregation is not None and isinstance(field.aggregation, str):
+                            string = '"' + field.aggregation + " " + field.column + " (" + field.sourceDataType + ", to " + field.internalDataType + ")" + '"'
+                        elif field.aggregation is not None and isinstance(field.aggregation, SQLFunction):
+                            string = '"' + field.aggregation.name + " " + field.column + " (" + field.sourceDataType + ", to " + field.internalDataType + ")" + '"'
+                        else:
+                            string = '"' + field.column + " (" + field.sourceDataType + ", to " + field.internalDataType + ")" + '"'
+                    else:
+                        if field.aggregation is not None and isinstance(field.aggregation, str):
+                            string = '"' + field.aggregation + " " + field.column + " (" + field.sourceDataType + ", to " + field.internalDataType + ") AS " + field.alias + '"'
+                        elif field.aggregation is not None and isinstance(field.aggregation, SQLFunction):
+                            string = '"' + field.aggregation.name + " " + field.column + " (" + field.sourceDataType + ", to " + field.internalDataType + ") AS " + field.alias + '"'
+                        else:
+                            string = '"' + field.column + " (" + field.sourceDataType + ", to " + field.internalDataType + ") AS " + field.alias + '"'
+                elif field.internalDataType.lower() == "date" and field.sourceDataType.lower() not in consistent_vec:
+                    # if internal and original data types are not consistent each other use this pattern
+                    # aggregation column (original data type, to internal data type, date_format YYYY-MM-DD ) AS Alias
+                    if field.sourceFormat is not None:
+                        if field.alias is None:
+                            if field.aggregation is not None and isinstance(field.aggregation, str):
+                                string = '"' + field.aggregation + " " + field.column + " (" + field.sourceDataType + ", to " + field.internalDataType + ", format " + field.sourceFormat + ")" + '"'
+                            elif field.aggregation is not None and isinstance(field.aggregation, SQLFunction):
+                                string = '"' + field.aggregation.name + " " + field.column + " (" + field.sourceDataType + ", to " + field.internalDataType + ", format " + field.sourceFormat + ")" + '"'
+                            else:
+                                string = '"' + field.column + " (" + field.sourceDataType + ", to " + field.internalDataType + ", format " + field.sourceFormat + ")" + '"'
+                        else:
+                            if field.aggregation is not None and isinstance(field.aggregation, str):
+                                string = '"' + field.aggregation + " " + field.column + " (" + field.sourceDataType + ", to " + field.internalDataType + ", format " + field.sourceFormat + ") AS " + field.alias + '"'
+                            elif field.aggregation is not None and isinstance(field.aggregation, SQLFunction):
+                                string = '"' + field.aggregation.name + " " + field.column + " (" + field.sourceDataType + ", to " + field.internalDataType + ", format " + field.sourceFormat + ") AS " + field.alias + '"'
+                            else:
+                                string = '"' + field.column + " (" + field.sourceDataType + ", to " + field.internalDataType + ", format " + field.sourceFormat + ") AS " + field.alias + '"'
+                    else:
+                        if field.alias is None:
+                            if field.aggregation is not None and isinstance(field.aggregation, str):
+                                string = '"' + field.aggregation + " " + field.column + " (" + field.sourceDataType + ", to " + field.internalDataType + ", format YYYY-MM-DD" + ")" + '"'
+                            elif field.aggregation is not None and isinstance(field.aggregation, SQLFunction):
+                                string = '"' + field.aggregation.name + " " + field.column + " (" + field.sourceDataType + ", to " + field.internalDataType + ", format YYYY-MM-DD" + ")" + '"'
+                            else:
+                                string = '"' + field.column + " (" + field.sourceDataType + ", to " + field.internalDataType + ", format YYYY-MM-DD" + ")" + '"'
+                        else:
+                            if field.aggregation is not None and isinstance(field.aggregation, str):
+                                string = '"' + field.aggregation + " " + field.column + " (" + field.sourceDataType + ", to " + field.internalDataType + ", format YYYY-MM-DD" + ") AS " + field.alias + '"'
+                            elif field.aggregation is not None and isinstance(field.aggregation, SQLFunction):
+                                string = '"' + field.aggregation.name + " " + field.column + " (" + field.sourceDataType + ", to " + field.internalDataType + ", format YYYY-MM-DD" + ") AS " + field.alias + '"'
+                            else:
+                                string = '"' + field.column + " (" + field.sourceDataType + ", to " + field.internalDataType + ", format YYYY-MM-DD" + ") AS " + field.alias + '"'
                 else:
-                    string = '"' + field.column + " (" + field.internalDataType + ") AS " + field.alias + '"'
+                    # else
+                    if field.alias is None:
+                        if field.aggregation is not None and isinstance(field.aggregation, str):
+                            string = '"' + field.aggregation + " " + field.column + " (" + field.sourceDataType + ")" + '"'
+                        elif field.aggregation is not None and isinstance(field.aggregation, SQLFunction):
+                            string = '"' + field.aggregation.name + " " + field.column + " (" + field.sourceDataType + ")" + '"'
+                        else:
+                            string = '"' + field.column + " (" + field.sourceDataType + ")" + '"'
+                    else:
+                        if field.aggregation is not None and isinstance(field.aggregation, str):
+                            string = '"' + field.aggregation + " " + field.column + " (" + field.sourceDataType + ") AS " + field.alias + '"'
+                        elif field.aggregation is not None and isinstance(field.aggregation, SQLFunction):
+                            string = '"' + field.aggregation.name + " " + field.column + " (" + field.sourceDataType + ") AS " + field.alias + '"'
+                        else:
+                            string = '"' + field.column + " (" + field.sourceDataType + ") AS " + field.alias + '"'
             fields.append(string)
         spell_str += ", ".join(fields) + "]"
 
